//-----------------------------------------------------------------------------
// EXT_MOD_KEY_MATRIX
// Version 1.0
// 05.12.2012 / UB
// "www.Mikrocontroller-4U.de"
// mc-4u(@)t-online.de
//-----------------------------------------------------------------------------
// Fuer ein externes Modul
// mit einer Tastatur-Matrix (mit maximal 64 Tasten)
//-----------------------------------------------------------------------------
// das File muss im Ordner
// "myAVR/WinAVR/AVR/Include/UB_EXT_MODULE/"
// liegen und mit #include <UB_EXT_MODULE/EXT_MOD_KEY_MATRIX.h>
// eingebunden werden
//-----------------------------------------------------------------------------
// Belegt : minimal = 2 Port-Bits  (1x1 = 1 Taste)
//          maximal = 16 Port-Bits (8x8 = 64 Tasten)
//          default = 6 Port-Bits  (4x2 = 8 Tasten)  [Port-D : 0 bis 5]
//-----------------------------------------------------------------------------
#include <stdio.h>


#define  KEY_XY_BIT_NC  9 // nicht benutzter Port-Bit (nicht aendern !!)


//-----------------------------------------------------------------------------
// KEY-Matrix : Spalten in X-Richtung (max=8)
//              1. DDR und PORT eintragen
//              2. Port-Bits von KEY_X_BIT_A bis KEY_X_BIT_H eintragen
//                 alle benutzten Bits = 0...7 -> kein Bit darf doppelt sein !
//                 alle nicht benutzten Bits = KEY_XY_BIT_NC
//-----------------------------------------------------------------------------

#define  KEY_X_DDR   DDRD  
#define  KEY_X_PORT  PORTD 

#define  KEY_X_BIT_A   0 // Port-D Bit-0
#define  KEY_X_BIT_B   1 // Port-D Bit-1
#define  KEY_X_BIT_C   2 // Port-D Bit-2
#define  KEY_X_BIT_D   3 // Port-D Bit-3
#define  KEY_X_BIT_E   KEY_XY_BIT_NC
#define  KEY_X_BIT_F   KEY_XY_BIT_NC
#define  KEY_X_BIT_G   KEY_XY_BIT_NC
#define  KEY_X_BIT_H   KEY_XY_BIT_NC




//-----------------------------------------------------------------------------
// KEY-Matrix : Zeilen in Y-Richtung (max=8)
//              1. DDR, PORT und PIN eintragen
//              2. Port-Bits von KEY_Y_BIT_A bis KEY_Y_BIT_H eintragen
//                 alle benutzten Bits = 0...7 -> kein Bit darf doppelt sein !
//                 alle nicht benutzten Bits = KEY_XY_BIT_NC
//-----------------------------------------------------------------------------

#define  KEY_Y_DDR   DDRD 
#define  KEY_Y_PORT  PORTD 
#define  KEY_Y_PIN   PIND  

#define  KEY_Y_BIT_A   4 // Port-D Bit-4
#define  KEY_Y_BIT_B   5 // Port-D Bit-5
#define  KEY_Y_BIT_C   KEY_XY_BIT_NC
#define  KEY_Y_BIT_D   KEY_XY_BIT_NC
#define  KEY_Y_BIT_E   KEY_XY_BIT_NC
#define  KEY_Y_BIT_F   KEY_XY_BIT_NC
#define  KEY_Y_BIT_G   KEY_XY_BIT_NC
#define  KEY_Y_BIT_H   KEY_XY_BIT_NC




#define  KEY_MATRIX_ENTPRELL   1000   // zum entprellen (1000= ca 5ms @16MHz)




//-----------------------------------------------------------------------------
// INFOS zur Benutzung :
// 01. Funktion "EXT_INIT_KEY_MATRIX" starten
// 02. Funktion "EXT_KEY_MATRIX_READ" prueft ob eine Taste gedrueckt ist
//-----------------------------------------------------------------------------




//-----------------------------------------------------------------------------
// Funktionen die vom MAIN benutzt werden koennen
//-----------------------------------------------------------------------------
void EXT_INIT_KEY_MATRIX(void);           // zum initialisieren der Tastatur Matrix
unsigned char EXT_KEY_MATRIX_READ(void);  // zum auslesen der Tastatur Matrix





//-----------------------------------------------------------------------------
// interne Funktionen
//-----------------------------------------------------------------------------
unsigned char key_matrix_read(void);
unsigned char key_matrix_read_y(void);


#define  KEY_MATRIX_PAUSE      20     // kleine Pause (nicht aendern)



//-----------------------------------------------------------------------------
void EXT_INIT_KEY_MATRIX(void) {	// zum initialisieren der Tastatur Matrix
  
  // Alle benutzten X-Pins auf Eingang
  if(KEY_X_BIT_A<8) KEY_X_DDR &= ~(1<<KEY_X_BIT_A);
  if(KEY_X_BIT_B<8) KEY_X_DDR &= ~(1<<KEY_X_BIT_B);
  if(KEY_X_BIT_C<8) KEY_X_DDR &= ~(1<<KEY_X_BIT_C);
  if(KEY_X_BIT_D<8) KEY_X_DDR &= ~(1<<KEY_X_BIT_D);
  if(KEY_X_BIT_E<8) KEY_X_DDR &= ~(1<<KEY_X_BIT_E);
  if(KEY_X_BIT_F<8) KEY_X_DDR &= ~(1<<KEY_X_BIT_F);
  if(KEY_X_BIT_G<8) KEY_X_DDR &= ~(1<<KEY_X_BIT_G);
  if(KEY_X_BIT_H<8) KEY_X_DDR &= ~(1<<KEY_X_BIT_H);

  // Alle Pull-Ups der X-Pins einschalten
  if(KEY_X_BIT_A<8) KEY_X_PORT |= (1<<KEY_X_BIT_A);
  if(KEY_X_BIT_B<8) KEY_X_PORT |= (1<<KEY_X_BIT_B);
  if(KEY_X_BIT_C<8) KEY_X_PORT |= (1<<KEY_X_BIT_C);
  if(KEY_X_BIT_D<8) KEY_X_PORT |= (1<<KEY_X_BIT_D);
  if(KEY_X_BIT_E<8) KEY_X_PORT |= (1<<KEY_X_BIT_E);
  if(KEY_X_BIT_F<8) KEY_X_PORT |= (1<<KEY_X_BIT_F);
  if(KEY_X_BIT_G<8) KEY_X_PORT |= (1<<KEY_X_BIT_G);
  if(KEY_X_BIT_H<8) KEY_X_PORT |= (1<<KEY_X_BIT_H);

  // Alle benutzten Y-Pins auf Eingang
  if(KEY_Y_BIT_A<8) KEY_Y_DDR &= ~(1<<KEY_Y_BIT_A);
  if(KEY_Y_BIT_B<8) KEY_Y_DDR &= ~(1<<KEY_Y_BIT_B);
  if(KEY_Y_BIT_C<8) KEY_Y_DDR &= ~(1<<KEY_Y_BIT_C);
  if(KEY_Y_BIT_D<8) KEY_Y_DDR &= ~(1<<KEY_Y_BIT_D);
  if(KEY_Y_BIT_E<8) KEY_Y_DDR &= ~(1<<KEY_Y_BIT_E);
  if(KEY_Y_BIT_F<8) KEY_Y_DDR &= ~(1<<KEY_Y_BIT_F);
  if(KEY_Y_BIT_G<8) KEY_Y_DDR &= ~(1<<KEY_Y_BIT_G);
  if(KEY_Y_BIT_H<8) KEY_Y_DDR &= ~(1<<KEY_Y_BIT_H);

  // Alle Pull-Ups der Y-Pins einschalten
  if(KEY_Y_BIT_A<8) KEY_Y_PORT |= (1<<KEY_Y_BIT_A);
  if(KEY_Y_BIT_B<8) KEY_Y_PORT |= (1<<KEY_Y_BIT_B);
  if(KEY_Y_BIT_C<8) KEY_Y_PORT |= (1<<KEY_Y_BIT_C);
  if(KEY_Y_BIT_D<8) KEY_Y_PORT |= (1<<KEY_Y_BIT_D);
  if(KEY_Y_BIT_E<8) KEY_Y_PORT |= (1<<KEY_Y_BIT_E);
  if(KEY_Y_BIT_F<8) KEY_Y_PORT |= (1<<KEY_Y_BIT_F);
  if(KEY_Y_BIT_G<8) KEY_Y_PORT |= (1<<KEY_Y_BIT_G);
  if(KEY_Y_BIT_H<8) KEY_Y_PORT |= (1<<KEY_Y_BIT_H);
}


//-----------------------------------------------------------------------------
// ret_wert : 0 = keine Taste betaetigt
// ret_wert : >0 = Matrix-Nummer der betaetigten Taste (1...64)
//-----------------------------------------------------------------------------
unsigned char EXT_KEY_MATRIX_READ(void) { // zum auslesen der Tastatur Matrix
  unsigned char ret_wert=0;
  static unsigned char old_key=0;
  static unsigned int pause=0;

  pause++;
  if(pause>=KEY_MATRIX_ENTPRELL) {
    // wenn pause zum entprellen vorbei
    pause=0;
    ret_wert=key_matrix_read();
    old_key=ret_wert;
  }
  else {
    ret_wert=old_key;
  }

  return(ret_wert);
}


//-----------------------------------------------------------------------------
unsigned char key_matrix_read(void) { // interne Funktion
  unsigned char ret_wert=0;  
  unsigned char wert;

  // XPOS = 0
  if(KEY_X_BIT_A<8) {
    KEY_X_DDR |= (1<<KEY_X_BIT_A);   // Pin auf Ausgang
    KEY_X_PORT &= ~(1<<KEY_X_BIT_A); // Bit auf Lo-Pegel
    wert=key_matrix_read_y();
    if(wert>0) ret_wert=0+wert;
    KEY_X_DDR &= ~(1<<KEY_X_BIT_A);  // Pin auf Eingang
    KEY_X_PORT |= (1<<KEY_X_BIT_A);  // Pull-Up
    if(ret_wert>0) return(ret_wert);
  }
  // XPOS = 1
  if(KEY_X_BIT_B<8) {
    KEY_X_DDR |= (1<<KEY_X_BIT_B);   // Pin auf Ausgang
    KEY_X_PORT &= ~(1<<KEY_X_BIT_B); // Bit auf Lo-Pegel
    wert=key_matrix_read_y();
    if(wert>0) ret_wert=8+wert;
    KEY_X_DDR &= ~(1<<KEY_X_BIT_B);  // Pin auf Eingang
    KEY_X_PORT |= (1<<KEY_X_BIT_B);  // Pull-Up
    if(ret_wert>0) return(ret_wert);
  }
  // XPOS = 2
  if(KEY_X_BIT_C<8) {
    KEY_X_DDR |= (1<<KEY_X_BIT_C);   // Pin auf Ausgang
    KEY_X_PORT &= ~(1<<KEY_X_BIT_C); // Bit auf Lo-Pegel
    wert=key_matrix_read_y();
    if(wert>0) ret_wert=16+wert;
    KEY_X_DDR &= ~(1<<KEY_X_BIT_C);  // Pin auf Eingang
    KEY_X_PORT |= (1<<KEY_X_BIT_C);  // Pull-Up
    if(ret_wert>0) return(ret_wert);
  }
  // XPOS = 3
  if(KEY_X_BIT_D<8) {
    KEY_X_DDR |= (1<<KEY_X_BIT_D);   // Pin auf Ausgang
    KEY_X_PORT &= ~(1<<KEY_X_BIT_D); // Bit auf Lo-Pegel
    wert=key_matrix_read_y();
    if(wert>0) ret_wert=24+wert;
    KEY_X_DDR &= ~(1<<KEY_X_BIT_D);  // Pin auf Eingang
    KEY_X_PORT |= (1<<KEY_X_BIT_D);  // Pull-Up
    if(ret_wert>0) return(ret_wert);
  }
  // XPOS = 4
  if(KEY_X_BIT_E<8) {
    KEY_X_DDR |= (1<<KEY_X_BIT_E);   // Pin auf Ausgang
    KEY_X_PORT &= ~(1<<KEY_X_BIT_E); // Bit auf Lo-Pegel
    wert=key_matrix_read_y();
    if(wert>0) ret_wert=32+wert;
    KEY_X_DDR &= ~(1<<KEY_X_BIT_E);  // Pin auf Eingang
    KEY_X_PORT |= (1<<KEY_X_BIT_E);  // Pull-Up
    if(ret_wert>0) return(ret_wert);
  }
  // XPOS = 5
  if(KEY_X_BIT_F<8) {
    KEY_X_DDR |= (1<<KEY_X_BIT_F);   // Pin auf Ausgang
    KEY_X_PORT &= ~(1<<KEY_X_BIT_F); // Bit auf Lo-Pegel
    wert=key_matrix_read_y();
    if(wert>0) ret_wert=40+wert;
    KEY_X_DDR &= ~(1<<KEY_X_BIT_F);  // Pin auf Eingang
    KEY_X_PORT |= (1<<KEY_X_BIT_F);  // Pull-Up
    if(ret_wert>0) return(ret_wert);
  }
  // XPOS = 6
  if(KEY_X_BIT_G<8) {
    KEY_X_DDR |= (1<<KEY_X_BIT_G);   // Pin auf Ausgang
    KEY_X_PORT &= ~(1<<KEY_X_BIT_G); // Bit auf Lo-Pegel
    wert=key_matrix_read_y();
    if(wert>0) ret_wert=48+wert;
    KEY_X_DDR &= ~(1<<KEY_X_BIT_G);  // Pin auf Eingang
    KEY_X_PORT |= (1<<KEY_X_BIT_G);  // Pull-Up
    if(ret_wert>0) return(ret_wert);
  }
  // XPOS = 7
  if(KEY_X_BIT_H<8) {
    KEY_X_DDR |= (1<<KEY_X_BIT_H);   // Pin auf Ausgang
    KEY_X_PORT &= ~(1<<KEY_X_BIT_H); // Bit auf Lo-Pegel
    wert=key_matrix_read_y();
    if(wert>0) ret_wert=56+wert;
    KEY_X_DDR &= ~(1<<KEY_X_BIT_H);  // Pin auf Eingang
    KEY_X_PORT |= (1<<KEY_X_BIT_H);  // Pull-Up
    if(ret_wert>0) return(ret_wert);
  }

  return(ret_wert);
}


//-----------------------------------------------------------------------------
unsigned char key_matrix_read_y(void) { // interne Funktion
  unsigned char ret_wert=0;
  unsigned char wert;
  unsigned int n;

  // kleine Pause
  for(n=0;n<KEY_MATRIX_PAUSE;n++) {
    // nur warten
  }


  // Y-Port einlesen
  wert=KEY_Y_PIN;

  // YPOS = 0
  if(KEY_Y_BIT_A<8) {
    if((wert & (1<<KEY_Y_BIT_A))==0) ret_wert=1;
    if(ret_wert>0) return(ret_wert);	
  }
  // YPOS = 1
  if(KEY_Y_BIT_B<8) {
    if((wert & (1<<KEY_Y_BIT_B))==0) ret_wert=2;
    if(ret_wert>0) return(ret_wert);
  }
  // YPOS = 2
  if(KEY_Y_BIT_C<8) {
    if((wert & (1<<KEY_Y_BIT_C))==0) ret_wert=3;
    if(ret_wert>0) return(ret_wert);
  }
  // YPOS = 3
  if(KEY_Y_BIT_D<8) {
    if((wert & (1<<KEY_Y_BIT_D))==0) ret_wert=4;
    if(ret_wert>0) return(ret_wert);
  }
  // YPOS = 4
  if(KEY_Y_BIT_E<8) {
    if((wert & (1<<KEY_Y_BIT_E))==0) ret_wert=5;
    if(ret_wert>0) return(ret_wert);
  }
  // YPOS = 5
  if(KEY_Y_BIT_F<8) {
    if((wert & (1<<KEY_Y_BIT_F))==0) ret_wert=6;
    if(ret_wert>0) return(ret_wert);
  }
  // YPOS = 6
  if(KEY_Y_BIT_G<8) {
    if((wert & (1<<KEY_Y_BIT_G))==0) ret_wert=7;
    if(ret_wert>0) return(ret_wert);
  }
  // YPOS = 7
  if(KEY_Y_BIT_H<8) {
    if((wert & (1<<KEY_Y_BIT_H))==0) ret_wert=8;
    if(ret_wert>0) return(ret_wert);
  }

  return(ret_wert);
}





